import os
from os.path import expanduser

home = expanduser("~")
goa_loader_root = f"{home}/data/goa-loader"
goa_loader_root = os.path.abspath(goa_loader_root)


def ensure_exists(path):
    if not os.path.exists(path):
        os.makedirs(path)


def get_base_dir():
    ensure_exists(goa_loader_root)
    ensure_exists(f"{goa_loader_root}/annotations")
    ensure_exists(f"{goa_loader_root}/images")
    return goa_loader_root



import os
from math import floor

import pandas as pd
import tensorflow as tf
from termcolor import colored

import goa_loader.download_data as download_lib
import goa_loader.util as util_lib
from goa_loader.path import get_base_dir


def load(
    base_dir=None,
    download=True,
    force_download=False,
    image_size=(200, 200),
    percent=100,
):
    base_dir = base_dir or get_base_dir()
    base_dir = os.path.abspath(base_dir)
    csv_file = f"{base_dir}/annotations/published_images.csv"

    if force_download:
        download_lib.download(base_dir=base_dir, percent=percent)
    if not os.path.exists(csv_file):
        if download:
            download_lib.download(base_dir=base_dir, percent=percent)
        else:
            raise ValueError(
                f"csv_file not found, {csv_file}. "
                "Try running `goa_loader.load(download=True)."
            )

    df = pd.read_csv(csv_file)
    df["local_path"] = df.apply(
        lambda row: util_lib.thumbnail_to_local(base_dir, row.iiifthumburl), axis=1
    )

    original_images = df.shape[0]
    indices = df.apply(lambda row: os.path.exists(row["local_path"]), axis=1)
    df = df[indices]
    new_images = df.shape[0]

    if new_images != original_images:
        print(
            colored(
                "Warning!  `goa_loader.load() detected missing images.`",
                "red",
                attrs=["bold"],
            )
        )
        print(f"\`goa_loader.load()` only found {new_images}/{original_images} images.")
        print(colored("To fix:", "yellow", attrs=["bold"]))
        print(
            "\tPass `force_download=True` to `goa_loader.load()` to attempt to "
            "re-download.  Alternatively, you can manually run: "
            "`goa_loader.download()`."
        )

    samples = floor(df.shape[0] * (percent / 100))
    print(f"Loading {samples}/{df.shape[0]} images")
    df = df.head(samples)
    ds = tf.data.Dataset.from_tensor_slices(df["local_path"])

    resize = tf.keras.layers.Resizing(image_size[0], image_size[1])

    def process_image(path):
        img = tf.io.read_file(path)
        img = tf.io.decode_jpeg(img, channels=3)
        return resize(img)

    ds = ds.map(process_image, num_parallel_calls=tf.data.AUTOTUNE)
    return ds



import argparse
import os
import urllib.request
from math import floor

import pandas as pd
import tqdm
from joblib import Parallel
from joblib import delayed

import goa_loader.util as util_lib
from goa_loader.path import get_base_dir

csv_remote_path = "https://raw.githubusercontent.com/NationalGalleryOfArt/opendata/main/data/published_images.csv"


def get_file(remote_url, out, timeout_seconds=10):
    with urllib.request.urlopen(remote_url, timeout=timeout_seconds) as response:
        with open(out, "wb") as out_file:
            data = response.read()  # a `bytes` object
            out_file.write(data)


def download(base_dir=None, percent=100):
    print("Downloading data...")

    base_dir = base_dir or get_base_dir()
    csv_file = f"{base_dir}/annotations/published_images.csv"

    if not os.path.exists(csv_file):
        print(f"CSV not found, downloading from {csv_remote_path}")
        get_file(csv_remote_path, out=csv_file, timeout_seconds=100)
        print("Download successful")

    print(f"Reading annotations from {csv_file}")
    df = pd.read_csv(csv_file)

    samples = floor(df.shape[0] * (percent / 100))
    print(f"Found {df.iiifthumburl.nunique()} images.")
    print(f"Downloading {samples}/{df.shape[0]} images")

    df = df.head(samples)
    print("Downloading images...")

    def download(thumb):
        out = util_lib.thumbnail_to_local(base_dir, thumb)
        if os.path.exists(out):
            return

        try:
            get_file(thumb, out=out)
        except Exception as e:
            print(e)
            print(f"failed to get {thumb}")

    Parallel(n_jobs=16)(
        delayed(download)(thumb) for thumb in tqdm.tqdm(df.iiifthumburl.unique())
    )
    print("Done downloading images")


def main():
    parser = argparse.ArgumentParser(description="download gallery of art data")
    parser.add_argument("--base_dir", "-b", default="./")

    args = parser.parse_args()
    if not args.base_dir:
        parser.print_help()
        quit()

    download(base_dir=args.base_dir)


if __name__ == "__main__":
    main()
